@@ -480,7 +480,7 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
     int_t    *perm_c; /* column permutation vector */
     int_t    *etree;  /* elimination tree */
     int_t    *colptr, *rowind;
-    int_t    colequ, Equil, factored, job, notran, rowequ;
+    int_t    Equil, factored, job, notran, colequ, rowequ;
     int_t    i, iinfo, j, irow, m, n, nnz, permc_spec, dist_mem_use;
     int      iam;
     int      ldx;  /* LDA for matrix X (global). */
@@ -624,9 +624,9 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
 		} else {
 		    if ( iinfo > 0 ) {
 			if ( iinfo <= m )
-			    fprintf(stderr, "The %d-th row of A is exactly zero\n", 
+			    fprintf(stderr, "The " IFMT "-th row of A is exactly zero\n", 
 				    iinfo);
-			else fprintf(stderr, "The %d-th column of A is exactly zero\n", 
+			else fprintf(stderr, "The " IFMT "-th column of A is exactly zero\n", 
 				     iinfo-n);
 			exit(-1);
 		    }
@@ -647,9 +647,11 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
 	    /* Equilibrate matrix A. */
 	    dlaqgs_dist(A, R, C, rowcnd, colcnd, amax, equed);
 	    if ( lsame_(equed, "R") ) {
-		ScalePermstruct->DiagScale = rowequ = ROW;
+		ScalePermstruct->DiagScale = ROW;
+		rowequ = ROW;
 	    } else if ( lsame_(equed, "C") ) {
-		ScalePermstruct->DiagScale = colequ = COL;
+		ScalePermstruct->DiagScale = COL;
+		colequ = COL;
 	    } else if ( lsame_(equed, "B") ) {
 		ScalePermstruct->DiagScale = BOTH;
 		rowequ = ROW;
@@ -693,7 +695,7 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
 
 	    if ( !iam ) {
 		/* Process 0 finds a row permutation for large diagonal. */
-		dldperm(job, m, nnz, colptr, rowind, a, perm_r, R1, C1);
+		dldperm_dist(job, m, nnz, colptr, rowind, a, perm_r, R1, C1);
 		
 		MPI_Bcast( perm_r, m, mpi_int_t, 0, grid->comm );
 		if ( job == 5 && Equil ) {
@@ -781,7 +783,7 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
 	t = SuperLU_timer_() - t;
 	stat->utime[ROWPERM] = t;
 #if ( PRNTlevel>=1 )
-	if ( !iam ) printf(".. LDPERM job %d\t time: %.2f\n", job, t);
+	if ( !iam ) printf(".. LDPERM job " IFMT "\t time: %.2f\n", job, t);
 #endif
     
     } else { /* options->RowPerm == NOROWPERM */
@@ -835,7 +837,7 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
 	if ( Fact != SamePattern_SameRowPerm ) {
 #if ( PRNTlevel>=1 ) 
 	    if ( !iam ) 
-		printf(".. symbfact(): relax %4d, maxsuper %4d, fill %4d\n",
+		printf(".. symbfact(): relax " IFMT ", maxsuper " IFMT ", fill " IFMT "\n",
 		       sp_ienv_dist(2), sp_ienv_dist(3), sp_ienv_dist(6));
 #endif
 	    t = SuperLU_timer_();
@@ -852,13 +854,13 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
 		QuerySpace_dist(n, -iinfo, Glu_freeable, &symb_mem_usage);
 #if ( PRNTlevel>=1 ) 
 		if ( !iam ) {
-		    printf("\tNo of supers %ld\n", Glu_persist->supno[n-1]+1);
-		    printf("\tSize of G(L) %ld\n", Glu_freeable->xlsub[n]);
-		    printf("\tSize of G(U) %ld\n", Glu_freeable->xusub[n]);
+		    printf("\tNo of supers %ld\n", (long long)Glu_persist->supno[n-1]+1);
+		    printf("\tSize of G(L) %ld\n", (long long)Glu_freeable->xlsub[n]);
+		    printf("\tSize of G(U) %ld\n", (long long)Glu_freeable->xusub[n]);
 		    printf("\tint %d, short %d, float %d, double %d\n", 
-			   sizeof(int_t), sizeof(short), sizeof(float),
-			   sizeof(double));
-		    printf("\tSYMBfact (MB):\tL\\U %.2f\ttotal %.2f\texpansions %d\n",
+			   (int) sizeof(int_t), (int) sizeof(short), 
+ 			   (int) sizeof(float), (int) sizeof(double));
+		    printf("\tSYMBfact (MB):\tL\\U %.2f\ttotal %.2f\texpansions " IFMT "\n",
 			   symb_mem_usage.for_lu*1e-6, 
 			   symb_mem_usage.total*1e-6,
 			   symb_mem_usage.expansions);
@@ -866,7 +868,7 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
 #endif
 	    } else {
 		if ( !iam ) {
-		    fprintf(stderr, "symbfact() error returns %d\n", iinfo);
+		    fprintf(stderr, "symbfact() error returns " IFMT "\n", iinfo);
 		    exit(-1);
 		}
 	    }
@@ -892,7 +894,7 @@ pdgssvx_ABglobal(superlu_options_t *options, SuperMatrix *A,
 	{
 	    int_t TinyPivots;
 	    float for_lu, total, max, avg, temp;
-	    dQuerySpace_dist(n, LUstruct, grid, &num_mem_usage);
+	    dQuerySpace_dist(n, LUstruct, grid, stat, &num_mem_usage);
 	    MPI_Reduce( &num_mem_usage.for_lu, &for_lu,
 		       1, MPI_FLOAT, MPI_SUM, 0, grid->comm );
 	    MPI_Reduce( &num_mem_usage.total, &total,
